# Create working directories in HDFS and local file system.
#
# Defining NO_REVOSHARE_DIRS as an environment variable before installing
# the "hadoop" package prevents those directories from being created.
#
# If REVOSHARE_DIRS_USER environment variable is defined while the package
# is installed:
#   * the "hadoop" command is run as this user
#   * a directory for this user is created in both local file system and HDFS
#
# The owner of the local top-level working directory is root. The owner of
# the HDFS working directory is either REVOSHARE_DIRS_USER or the first user
# for whom this directory can be created: root, hdfs, mapr



# TODO There are a number of security considerations:
#        * should we create these dirs for user X but run commands as user Y
#        * should we run chown on these commands if REVOSHARE_DIRS_USER is not root?
#        * should we introduce a new group "microsoftr" that owns those working
#          directories and can be used to restrict access to them (that would
#          allow us to change access from 0777 to 0770)




# Create the default RevoShare hadoop directories.
#
# In case of a failure print a warning and exit with a non-zero return code.
#
function create_work_directories {
  if [ -n "${NO_REVOSHARE_DIRS}" ] ; then
    return 0
  fi

  if [ -n "${REVOSHARE_DIRS_USER}" ] ; then
    create_all_with_custom_user
    return $?
  fi

  # if the user has not been specified run all as root
  mkdir -p /var/RevoShare && chmod 777 /var/RevoShare

  if [ $? -ne 0 ] ; then
    # TODO clean-up?
    error "creating default local directories failed"
  fi

  if [[ -n "${HADOOP_BIN}" ]] ; then 
      try_create_hdfs "root" || try_create_hdfs "hdfs" || try_create_hdfs "mapr"
  fi

  # TODO clean-up?
  if [ $? -ne 0 ] ; then
    echo "Failure while creating default hadoop directories." >&2
    return 1
  fi

  return $?
}



# Create all working directories as user REVOSHARE_DIRS_USER
#
function create_all_with_custom_user {
  rc=0

  create_hdfs_dir "${REVOSHARE_DIRS_USER}" "/user" "755" && \
    create_hdfs_dir "${REVOSHARE_DIRS_USER}" "/user/RevoShare" "777" && \
    create_hdfs_dir "${REVOSHARE_DIRS_USER}" "/user/RevoShare/${REVOSHARE_DIRS_USER}" "777"

  if [ $? -ne 0 ] ; then
    error "creating default hadoop directories failed for specified user ${REVOSHARE_DIRS_USER}"
    rc=1
  fi

  mkdir -p /var/RevoShare/${REVOSHARE_DIRS_USER} && \
    chmod 777 /var/RevoShare && \
    chmod 777 /var/RevoShare/${REVOSHARE_DIRS_USER}

  if [ $? -ne 0 ] ; then
    error "creating default local directories failed for specified user ${REVOSHARE_DIRS_USER}"
    rc=1
  fi

  # TODO clean-up if failed?
  return ${rc}
}


# Try creating HDFS working directory as the given user.
#
#  $1 - user name to run "hadoop" as
#
function try_create_hdfs {
  local USER="$1"
  create_hdfs_dir "${USER}" "/user" "755" && \
  create_hdfs_dir "${USER}" "/user/RevoShare" "777"
  return $?
}


# Create HDFS working directory and set access flags.
#
#  $1 - Name of the user to run "hadoop" as
#  $2 - Directory path
#  $3 - Access flags for "chmod"
#
function create_hdfs_dir {
  local USER="$1"
  local DIR="$2"
  local PERMISSIONS="$3"

  if [ "${USER}" == "root" ] ; then
    "${HADOOP_BIN}" fs -mkdir -p "${DIR}" 2> /dev/null
    "${HADOOP_BIN}" fs -chmod "${PERMISSIONS}" "${DIR}" 2> /dev/null
    rc=$?
  else
    sudo -u "${USER}" "${HADOOP_BIN}" fs -mkdir -p "${DIR}" 2> /dev/null
    sudo -u "${USER}" "${HADOOP_BIN}" fs -chmod "${PERMISSIONS}" "${DIR}" 2> /dev/null
    rc=$?
  fi

  return ${rc}
}
