# Discover Java that Microsoft R Server should use.
#
# Required variable: INSTALL_PREFIX
#   INSTALL_PREFIX needs to be defined in the master script; it should
#   point to the top directory of Microsoft R Server, e.g.
#   INSTALL_PREFIX=/usr/lib64/microsoft-r/3.3/
#


# This is the library we're searching for and that's to be symlinked in
# INSTALL_PREFIX/hadoop/libjvm.so
#
LIBJVM_SO_NAME="libjvm.so"


# Attempt to find libjvm.so and create a symlink that Microsoft R Server
# can use.
#
# $1 - Package system type: RPM or DEB
# $2 - The path to the requested symlink.
#
function create_libjvm_symlink {
  local PKG_TYPE="$1"
  local LIBJVM_SYMLINK_PATH="$2"

  LIBJVMSO_PATH=$(find_libjvm "${PKG_TYPE}")
  if [ -z "${LIBJVMSO_PATH}" ] ; then
    echo "could not find ${LIBJVM_SO_NAME} in the system" >&2
    return
  fi

  ln -s "${LIBJVMSO_PATH}" "${LIBJVM_SYMLINK_PATH}"

  return $?
}




# Attempt to find libjvm.so.
#
# $1 - Package system type: RPM or DEB
#
function find_libjvm {
  local PKG_TYPE="$1"

  # try JAVA_HOME first
  LIBJVM_PATH=$(libjvm_from_java_home)
  if [ -n "${LIBJVM_PATH}" ] && at_least_java7 "${LIBJVM_PATH}"; then
    echo "${LIBJVM_PATH}"
    return
  fi

  # if couldn't be found, try other heuristics
  echo -e "Could not extract JAVA_HOME from yarn/hadoop.\nAttempting at discovering it through system settings." >&2

  # first try through alternatives
  ALTERNATIVES_JVMSO=$(find_by_pkg "${PKG_TYPE}" $(readlink -nf /etc/alternatives/jre))
  if [ -n "${ALTERNATIVES_JVMSO}" ] && at_least_java7 "${ALTERNATIVES_JVMSO}" ; then
    echo "${ALTERNATIVES_JVMSO}"
    return
  fi

  # second, try through java command
  JAVA_PATH="$(readlink -nf $(which java 2>/dev/null))"
  JAVA_CMDLINE_JVMSO=$(find_by_pkg "${PKG_TYPE}" "${JAVA_PATH}")
  if [ -n "${JAVA_CMDLINE_JVMSO}" ] && at_least_java7 "${JAVA_CMDLINE_JVMSO}" ; then
    echo "${JAVA_CMDLINE_JVMSO}"
    return
  fi

  # finally, try through /usr/java which might be defined in some systems
  USR_JAVA_JVMSO=$(find_by_pkg "${PKG_TYPE}" /usr/java)
  if [ -n "${USR_JAVA_JVMSO}" ] && at_least_java7 "${USR_JAVA_JVMSO}" ; then
    echo "${USR_JAVA_JVMSO}"
    return
  fi

  # if nothing worked, try a global search
  ALL_LIBJVM=($(find / -type f -name "${LIBJVM_SO_NAME}" ! -path "/tmp/*" 2>/dev/null))

  if [ ${#ALL_LIBJVM[@]} -eq 1 ] ; then
    if at_least_java7 "${USR_JAVA_JVMSO}"; then
      echo "found a single ${LIBJVM_SO_NAME} in filesystem, going to use it for Microsoft R Server" >&2
      echo "${ALL_LIBJVM[0]}"
    else
      echo "The only Java present in the system is too old. Java >= 1.7 is required" >&2
    fi
    return
  fi

  if [ ${#ALL_LIBJVM[@]} -eq 0 ] ; then
    echo "no ${LIBJVM_SO_NAME} found in the system" >&2
  else
    # install the first that is at least 1.7
    for candidate in "${ALL_LIBJVM[@]}"; do
      if at_least_java7 "${candidate}"; then
        echo "Multiple ${LIBJVM_SO_NAME} files found in the system but none clearly associated with Yarn/Hadoop" >&2
        echo "Microsoft R Server is going to interface with Java with using ${ALL_LIBJVM[@]}" >&2
        echo "You can also re-install the package with JAVA_HOME defined in the environment" >&2
        echo "${ALL_LIBJVM[@]}"
        return
      fi
    done
  fi
}


# Attempts at extracting JAVA_HOME from a number of known sources.
#
function find_java_home {
  # Try our JAR-bases solution first; it will output the variable name
  # and the equal sign (`=`) together with the value.
  local YARN_VAR="$(yarn jar "${HADOOP_PATH}/jar/HadoopEnv-1.0-SNAPSHOT.jar" \
                    com.revolutionanalytics.hadooputils.HadoopEnv JAVA_HOME 2>/dev/null | grep JAVA_HOME )"
  YARN_VAR="${YARN_VAR#JAVA_HOME=}"

  if [ -n "${YARN_VAR}" ] ; then
    echo "${YARN_VAR}"
    return
  fi

  # if that didn't work, try Cloudera's API
  CLOUDERA_VAR="$(/usr/share/cmf/bin/cmf-server 2>/dev/null | grep JAVA_HOME)"
  CLOUDERA_VAR="${CLOUDERA_VAR#JAVA_HOME=}"
  if [ -n "${CLOUDERA_VAR}" ] ; then
    echo "${CLOUDERA_VAR}"
    return
  fi

  # maybe just a system-wide JAVA_HOME?...
  echo "${JAVA_HOME}"
}



# Attempt to find libjvm.so based on Yarn/Hadoop settings or system-wide
# JAVA_HOME setting. If successful, a full path to libjvm.so will be
# returned via stdout.
#
function libjvm_from_java_home {
  # first try to discover Java home
  JAVA_HOME="$(find_java_home)"

  # Java home is set to something - see if we can find libjvm.so there
  LIKELY_JVMSO_PATH="${JAVA_HOME}/jre/lib/amd64/server/${LIBJVM_SO_NAME}"
  if [ -f "${LIKELY_JVMSO_PATH}" ] ; then
    echo "${LIKELY_JVMSO_PATH}"
    return
  fi

  # if the standard path didn't work try finding the file
  CANDIDATE_JVMSO=($(find "${JAVA_HOME}" -type f -name ${LIBJVM_SO_NAME} 2>/dev/null))

  # if nothing found return now
  if [ ${#CANDIDATE_JVMSO[@]} -eq 0 ] ; then
    return
  fi

  # there should be a single *.so found; if more than one, warn
  if [ ${#CANDIDATE_JVMSO[@]} -ge 1 ] ; then
    echo "more than one ${LIBJVM_SO_NAME} found under JAVA_HOME: ${CANDIDATE_JVMSO[@]}" >&2
    echo "Microsoft R Server is going to use the first one" >&2
  fi

  echo "${CANDIDATE_JVMSO[0]}"
}



# Finds the package that owns a file given as the argument.
#
# $1 - Package system type: RPM or DEB
# $2 - Full file/directory path.
#
# Run "rpm -qa" and not just "rpm -q" if you want to support patterns
# (globs).
#
function find_by_pkg {
  local PKG_TYPE="$1"
  local FILE="$2"

  if [ "${PKG_TYPE}" == "rpm" -o "${PKG_TYPE}" == "RPM" ] ; then
    # see if there's a package that this file/directory belongs to
    PKG=$(rpm -qf "${FILE}" --queryformat '%{NAME}\n' 2>/dev/null)
    if [ -z "${PKG}" ] ; then
      return
    fi

    # see if this package contains a libjvm.so
    JVM_PATH=$(rpm -ql "${PKG}" | grep "${LIBJVM_SO_NAME}\$")
    if [ -n "${JVM_PATH}" ] ; then
      echo "${JVM_PATH}"
    fi

  elif [ "${PKG_TYPE}" == "deb" -o "${PKG_TYPE}" == "DEB" ] ; then
    # see if there's a package that this file/directory belongs to
    PKG=$(dpkg --search "${FILE}" 2>/dev/null | sed -e 's/:.*//')
    if [ -z "${PKG}" ] ; then
      return
    fi

    # see if this package contains a libjvm.so
    JVM_PATH=$(dpkg --listfiles "${PKG}" | grep "${LIBJVM_SO_NAME}\$")
    if [ -n "${JVM_PATH}" ] ; then
      echo "${JVM_PATH}"
    fi

  else
    echo "unknown package type ${PKG_TYPE}" >&2
    return
  fi
}


# Make sure Java is at least in version 1.7
#
# $1 Path to libjvm.so
#
# Returns: true if at least 1.7, false otherwise.
#
function at_least_java7 {
  local LIBJVM="$1"
  local JAVAPATH=$(dirname "${LIBJVM}")
  local JAVABIN="${JAVAPATH}/../../../bin/java"

  # if prospectable "java" is not executable (or does not exist)
  # attempt to find it stepping out from the current dir recursively
  if [ ! -x "${JAVABIN}" ]; then
    local WD="${PWD}"
    cd "${JAVAPATH}"
    while true; do
      JAVABIN=$(find . -name java 2>/dev/null)
      if [ -n "${JAVABIN}" ]; then
         JAVABIN="$(readlink -nf "${JAVABIN}")"
         break
      fi
      cd ..
    done
    cd "${WD}"
  fi

  VERSION_STRING="$("${JAVABIN}" -version 2>&1)"
  echo "${VERSION_STRING}" | grep '1\.[7-9]' 1>/dev/null 2>&1

  return $?
}
